@@ -8,6 +8,7 @@ fi
 FULL_PATH=$(realpath "${BASH_SOURCE[0]}")
 CONFIG_FILE=${CONFIG_FILE:-"/etc/default/ovs-dpdk.conf"}
 SERVICE_FILE="/etc/systemd/system/ovs-dpdkctl.service"
+BRIDGE_SERVICE_FILE="/etc/systemd/system/ovs-dpdk-bridge.service"
 
 function get_value {
     crudini --get $CONFIG_FILE $@
@@ -37,8 +38,12 @@ function del_config {
     rm -f $CONFIG_FILE
 }
 
+function is_redhat_family {
+    [[ -e /etc/redhat-release ]] ; echo $?
+}
+
 function generate_pciwhitelist {
-    local _Whitelist=''
+    _Whitelist=''
     for nic in $(list_dpdk_nics); do
         address="$(get_value $nic address)"
         if [ "$_Whitelist" == '' ]; then
@@ -79,6 +84,7 @@ function gen_config {
     set_value ovs ovs_socket_mem ${ovs_socket_mem:-"512"}
     set_value ovs dpdk_interface_driver ${dpdk_interface_driver:-"uio_pci_generic"}
     set_value ovs hugepage_mountpoint ${hugepage_mountpoint:-"/dev/hugepages"}
+    set_value ovs physical_port_policy ${ovs_physical_port_policy:-"named"}
 
     ls -al /sys/class/net/* | awk '$0 ~ /pci/ {n=split($NF,a,"/"); print "\n[" a[n] "]\naddress = " a[n-2]  "\ndriver ="}' >> $CONFIG_FILE
 
@@ -88,23 +94,21 @@ function gen_config {
     for nic in $(list_dpdk_nics); do
         set_value $nic driver ${dpdk_interface_driver:-"uio_pci_generic"}
     done
-    set_value ovs pci_whitelist "'${pci_whitelist:-$(generate_pciwhitelist)}'"
-
+    set_value ovs pci_whitelist "${pci_whitelist:-$(generate_pciwhitelist)}"
 }
 
 function bind_nic {
-    echo $1 > /sys/bus/pci/drivers/$2/bind
     echo $2 > /sys/bus/pci/devices/$1/driver_override
+    echo $1 > /sys/bus/pci/drivers/$2/bind
 }
 
 function unbind_nic {
     echo $1 > /sys/bus/pci/drivers/$2/unbind
     echo > /sys/bus/pci/devices/$1/driver_override
-
 }
 
 function list_dpdk_nics {
-    for nic in $(crudini --get $CONFIG_FILE ovs port_mappings | cut -d : -f 1); do
+    for nic in $(get_value ovs port_mappings | tr ',' '\n' | cut -d : -f 1); do
         echo $nic;
     done
 }
@@ -137,7 +141,6 @@ function unbind_nics {
     done
 }
 
-
 function get_address_by_name {
     ls -al /sys/class/net/$1 | awk '$0 ~ /pci/ {n=split($NF,a,"/"); print a[n-2] }'
 }
@@ -147,7 +150,7 @@ function get_driver_by_address {
 }
 
 function get_port_bridge {
-    for pair in $(get_value ovs port port_mappings); do
+    for pair in $(get_value ovs port_mappings | tr ',' '\n'); do
         nic=`echo $pair | cut -f 1 -d ":"`
         if [[ "$nic" == "$1" ]]; then
             bridge=`echo $pair | cut -f 2 -d ":"`
@@ -178,11 +181,11 @@ function init_ovs_bridges {
 }
 
 function init_ovs_interfaces {
-    local pci_port_pairs ==''
+    pci_port_pairs=''
     for nic in $(list_dpdk_nics); do
         address="$(get_value $nic address)"
         if [ "$pci_port_pairs" == '' ]; then
-            pci_port_pairs ="$address,$nic"
+            pci_port_pairs="$address,$nic"
         else
             pci_port_pairs="$pci_port_pairs $address,$nic"
         fi
@@ -194,14 +197,16 @@ function init_ovs_interfaces {
         nic="$(echo $pair | cut -f 2 -d ",")"
         bridge="$(get_port_bridge $nic)"
         # ovs 2.6 and older requires dpdkX names, ovs 2.7+ requires dpdk-devargs instead.
-        ovs-vsctl --no-wait --may-exist add-port $bridge "dpdk${dpdk_port_number}" \
-        -- set Interface  "dpdk${dpdk_port_number}" type=dpdk ||  \
-        ovs-vsctl --no-wait --may-exist add-port $bridge $nic \
-        -- set Interface  $nic type=dpdk options:dpdk-devargs=$addr
+        if [ "$(get_value ovs physical_port_policy)" == "indexed" ]; then
+            ovs-vsctl --no-wait --may-exist add-port $bridge "dpdk${dpdk_port_number}" \
+            -- set Interface  "dpdk${dpdk_port_number}" type=dpdk
+        else
+            ovs-vsctl --may-exist add-port $bridge $nic \
+            -- set Interface  $nic type=dpdk options:dpdk-devargs=$addr
+        fi
 
         dpdk_port_number=$((dpdk_port_number+1))
     done
-
 }
 
 function init {
@@ -210,6 +215,58 @@ function init {
     init_ovs_interfaces
 }
 
+function install_network_manager_conf {
+    pair=$(get_value ovs cidr_mappings)
+    bridge=`echo $pair | cut -f 1 -d ":"`
+    cidr=`echo $pair | cut -f 2 -d ":"`
+    ip=`echo $cidr | cut -f 1 -d "/"`
+    prefix=`echo $cidr | cut -f 2 -d "/"`
+    mask=""
+    full_octets=$(expr $prefix / 8)
+    partial_octet=$(expr $prefix % 8)
+
+    for octet in 0 1 2 3 ; do
+        if [[ "$octet" < "$full_octets" ]]; then
+            mask+=255
+        elif [[ "$octet" == "$full_octets" ]]; then
+            mask+=$((256 - 2**(8-$partial_octet)))
+        else
+            mask+=0
+        fi
+        [[ "$octet" < 3 ]] && mask+=.
+    done
+    if  [[ $(is_redhat_family) == 0 ]]; then
+        cat << EOF | tee "/etc/sysconfig/network-scripts/ifcfg-$bridge"
+DEVICE=$bridge
+BOOTPROTO=static
+IPADDR=$ip
+NETMASK=$mask
+HOTPLUG=yes
+ONBOOT=yes
+NM_CONTROLLED=no
+EOF
+install_redhat_bridge_service $bridge
+    else
+        cat << EOF | tee "/etc/network/interfaces.d/$bridge.cfg"
+    auto $bridge
+    iface $bridge inet static
+        address $ip
+        netmask $mask
+EOF
+
+    fi
+}
+
+function uninstall_network_manager_conf {
+    pair=$(get_value ovs cidr_mappings)
+    bridge=`echo $pair | cut -f 1 -d ":"`
+    if  [[ $(is_redhat_family) == 0 ]]; then
+        rm -f /etc/sysconfig/network-scripts/ifcfg-$bridge
+    else
+        rm -f /etc/network/interfaces.d/$bridge.cfg
+    fi
+}
+
 function install_service {
     cat << EOF | tee "$SERVICE_FILE"
 
@@ -235,13 +292,63 @@ EOF
     systemctl enable ovs-dpdkctl
 }
 
+function install_redhat_bridge_service {
+    cat << EOF | tee "$BRIDGE_SERVICE_FILE"
+[Unit]
+Description=configuration service for ovs-dpdk bridge.
+After=docker.service
+After=network.target
+After=ovs-dpdkctl.service
+
+[Service]
+Type=simple
+RemainAfterExit=yes
+ExecStartPre=/bin/bash -c "[[ -e /sys/class/net/$1 ]]"
+ExecStart=/usr/sbin/ifup $1
+ExecStop=/usr/sbin/ifdown $1
+Restart=on-failure
+RestartSec=5
+
+[Install]
+WantedBy=multi-user.target
+
+EOF
+    systemctl daemon-reload
+    systemctl enable ovs-dpdk-bridge
+}
+
 function uninstall_service {
     systemctl disable ovs-dpdkctl
     rm -f "$SERVICE_FILE"
+    if [ -e "$BRIDGE_SERVICE_FILE" ]; then
+        systemctl disable ovs-dpdk-bridge
+        rm -f "$BRIDGE_SERVICE_FILE"
+    fi
     systemctl daemon-reload
 }
 
+function configure_kernel_modules {
+    driver="$(get_value ovs dpdk_interface_driver)"
+    lsmod | grep -ws $driver > /dev/null || modprobe $driver
+    if  [[ $(is_redhat_family) == 0 ]]; then
+        [[ ! -e /etc/modules-load.d/${driver}.conf ]] && echo $driver | tee /etc/modules-load.d/${driver}.conf
+    else
+        grep -ws $driver /etc/modules > /dev/null || echo $driver | tee -a /etc/modules
+    fi
+}
+
+function unconfigure_kernel_modules {
+    driver="$(get_value ovs dpdk_interface_driver)"
+    lsmod | grep -ws $driver > /dev/null && rmmod $driver
+    if [[ $(is_redhat_family) == 0 ]] ; then
+        [[ -e /etc/modules-load.d/${driver}.conf ]] && rm -f /etc/modules-load.d/${driver}.conf
+    else
+        grep  -ws $driver /etc/modules > /dev/null && sed -e "s/$driver//" -i /etc/modules
+    fi
+}
+
 function install {
+    configure_kernel_modules
     if [ ! -e "$SERVICE_FILE" ]; then
         install_service
     fi
@@ -253,6 +360,10 @@ function install {
         gen_config
     fi
     systemctl start ovs-dpdkctl
+    install_network_manager_conf
+    if  [[ $(is_redhat_family) == 0 ]]; then
+        systemctl start ovs-dpdk-bridge
+    fi
 }
 
 function uninstall {
@@ -260,6 +371,8 @@ function uninstall {
     if [ -e "$SERVICE_FILE" ]; then
         uninstall_service
     fi
+    uninstall_network_manager_conf
+    unconfigure_kernel_modules
     if [ -e /bin/ovs-dpdkctl ]; then
         rm -f /bin/ovs-dpdkctl
     fi
@@ -268,8 +381,7 @@ function uninstall {
     fi
 }
 
-
-function useage {
+function usage {
     cat << "EOF"
 ovs-dpdkctl.sh: A tool to configure ovs with dpdk.
 
@@ -302,27 +414,27 @@ commands:
       ovs-dpdkctl config.
     - creates dpdk ports as defined by ovs port_mappings in
       ovs-dpdkctl config.
-  - useage:
+  - usage:
     - prints this message
 
 options:
   - debuging:
     - To enable debuging export OVS_DPDK_CTL_DEBUG=True
   - install:
-    - The varibles discribed below can be defined to customise
+    - The varibles described below can be defined to customise
       installation of ovs-dpdkctl.
       <variable>=<value> ovs-dpdkctl.sh install
     - bridge_mappings:
-      - A comma seperated list of physnet to bridge mappings.
+      - A comma separated list of physnet to bridge mappings.
       - Example: bridge_mappings=physnet1:br-ex1,physnet2:br-ex2
       - Default: ""
     - port_mappings:
-      - A comma seperated list of port to bridge mappings.
+      - A comma separated list of port to bridge mappings.
       - Example: port_mappings=eth1:br-ex1,eth2:br-ex2
       - Default: generated form bridge_mappings assuming bridge names
                  are constructed by appending br- to port name.
     - cidr_mappings:
-      - A comma seperated list of bridge to cidr mappings.
+      - A comma separated list of bridge to cidr mappings.
       - Example: cidr_mappings=br-ex1:192.168.1.1/24,br-ex2:192.168.2.1/24
       - Default: ""
     - ovs_coremask:
@@ -336,8 +448,8 @@ options:
       - A hex encoded string container a bitmask of what cpu cores
         to pin the dataplane pool mode driver treads of the ovs-vswitchd to.
       - Each bit set in the bitmask will result in the creating of a pmd.
-      - For best performance it is recomended to allocate at least 1 pmd per
-        numa node. On systems with HyperThreading enabled it is recomended to also
+      - For best performance it is recommended to allocate at least 1 pmd per
+        numa node. On systems with HyperThreading enabled it is recommended to also
         allocate the HT sibling core in the pmd_coremask.cores allocated
         to ovs with dpdk via the pmd_coremask should be removed from the
         nova vcpu_pin_set and isolated from the kernel scheduler.
@@ -350,7 +462,7 @@ options:
       - Example: ovs_mem_channels=2
       - Default: "4"
     - ovs_socket_mem:
-      - A comma seperated list of hugepage memory, specifed in MBs per numa node,
+      - A comma separated list of hugepage memory, specifed in MBs per numa node,
         allocated to the ovs-vswitchd to use for the dpdk dataplane.
       - For best performance memory should be allocated evenly across all numa node
         that will run a pmd.
@@ -365,7 +477,7 @@ options:
       - Example: dpdk_interface_driver=vfio_pci
       - Default: "uio_pci_generic"
     - pci_whitelist:
-      - A repeated space seperated list of pci whitelist flags
+      - A repeated space separated list of pci whitelist flags
         for allowed ovs-dpdk ports.
       - The pci_whitelist allows multiple dpdk primary process to
         utilise different pci devices without resulting in a conflict
@@ -380,7 +492,7 @@ if [ $# -ge  1 ]; then
     func=$1
     shift
 else
-    func="useage"
+    func="usage"
 fi
 
 #replace with switch later
